| package |
package := Package name: 'JSON'.
package paxVersion: 1;
	basicComment: ''.


package classNames
	add: #JsonReader;
	add: #JsonSyntaxError;
	add: #JsonTestCase;
	yourself.

package methodNames
	add: #Boolean -> #jsonWriteOn:;
	add: #Dictionary -> #jsonWriteOn:;
	add: #Interval -> #jsonWriteOn:;
	add: #Number -> #jsonWriteOn:;
	add: #Object -> #asJson;
	add: #Object -> #jsonInstVarNames;
	add: #Object -> #jsonWriteOn:;
	add: #SequenceableCollection -> #jsonWriteOn:;
	add: #String -> #jsonWriteOn:;
	add: #UndefinedObject -> #asJson;
	add: #UndefinedObject -> #jsonWriteOn:;
	add: 'Object class' -> #jsonInstVarNames;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Camp Smalltalk\SUnit\SUnit';
	yourself).

package!

"Class Definitions"!

Object subclass: #JsonReader
	instanceVariableNames: 'stream'
	classVariableNames: 'CharacterEscapeMap'
	poolDictionaries: ''
	classInstanceVariableNames: ''!
Error subclass: #JsonSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TestCase subclass: #JsonTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!Boolean methodsFor!

jsonWriteOn: aStream
	self printOn: aStream! !
!Boolean categoriesFor: #jsonWriteOn:!json!public! !

!Dictionary methodsFor!

jsonWriteOn: aStream
	aStream nextPutAll: '{'.
	self keysAndValuesDo: [:key :value |
		key jsonWriteOn: aStream.
		aStream nextPutAll: ':'.
		value jsonWriteOn: aStream.
		aStream nextPutAll: ','].
	self notEmpty ifTrue: [aStream skip: -1].
	aStream nextPutAll: '}'
! !
!Dictionary categoriesFor: #jsonWriteOn:!json!public! !

!Interval methodsFor!

jsonWriteOn: aStream
	aStream nextPutAll: '['.
	start printOn: aStream.
	aStream nextPutAll: ','.
	stop printOn: aStream.
	aStream nextPutAll: ']'! !
!Interval categoriesFor: #jsonWriteOn:!json!public! !

!Number methodsFor!

jsonWriteOn: aStream
	self printOn: aStream! !
!Number categoriesFor: #jsonWriteOn:!json!public! !

!Object methodsFor!

asJson
	| stream |
	stream := WriteStream on: String new.
	self jsonWriteOn: stream.
	^stream contents!

jsonInstVarNames
	"override if we want to provide inst level customization"

	^self class jsonInstVarNames!

jsonWriteOn: stream
	| count vars |
	stream nextPut: ${.
	count := 1.
	vars := self jsonInstVarNames.
	vars do: [:each | | val msg |
		msg := each asSymbol.
		stream nextPutAll: '"', each, '"'.
		stream nextPut: $:.
		val := (self respondsTo: msg)
			ifTrue: [self perform: msg]
			ifFalse: [self instVarAt: count].
		count := count + 1.
		val jsonWriteOn: stream.
		each = vars last
			ifFalse: [stream nextPut: $,]].
	stream nextPut: $}.
	^stream contents! !
!Object categoriesFor: #asJson!json!public! !
!Object categoriesFor: #jsonInstVarNames!json!public! !
!Object categoriesFor: #jsonWriteOn:!json!public! !

!Object class methodsFor!

jsonInstVarNames
	"override to customize what gets passed"

	^self allInstVarNames! !
!Object class categoriesFor: #jsonInstVarNames!json!public! !

!SequenceableCollection methodsFor!

jsonWriteOn: aStream
	aStream nextPutAll: '['.
	self do: [:each | each jsonWriteOn: aStream] separatedBy: [aStream nextPutAll: ','].
	aStream nextPutAll: ']'! !
!SequenceableCollection categoriesFor: #jsonWriteOn:!json!public! !

!String methodsFor!

jsonWriteOn: aStream
	| replacement |
	aStream nextPut: $".
	self do: [:character |
		(replacement := JsonReader escapeForCharacter: character)
			ifNil: [ aStream nextPut: character ]
			ifNotNil: [ aStream nextPut: $\; nextPut: replacement ].
	].
	aStream nextPut: $"! !
!String categoriesFor: #jsonWriteOn:!json!public! !

!UndefinedObject methodsFor!

asJson
	| stream |
	stream := WriteStream on: String new.
	self jsonWriteOn: stream.
	^stream contents!

jsonWriteOn: aWriteStream 
	aWriteStream nextPutAll: 'null'! !
!UndefinedObject categoriesFor: #asJson!public! !
!UndefinedObject categoriesFor: #jsonWriteOn:!json!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

JsonReader guid: (GUID fromString: '{6B48199F-D62C-4FED-A028-7DF420D73EDE}')!
JsonReader comment: ''!
!JsonReader categoriesForClass!Kernel-Objects! !
!JsonReader methodsFor!

consume: aString returning: anObject
	aString do: [:c | self next == c ifFalse: [JsonSyntaxError new raiseSignal: 'Expected ''', aString, ''''] ].
	^ anObject!

interpretStringEscape
	
	| c |
	c := self next.
	c == $b ifTrue: [^Character backspace].
	c == $n ifTrue: [^Character lf].
	c == $f ifTrue: [^Character newPage].
	c == $r ifTrue: [^Character cr].
	c == $t ifTrue: [^Character tab].
	c == $u
		ifTrue:
			[| valueStream |
			valueStream := (self stream next: 4) readStream.
			^Character value: (Integer readFrom: valueStream radix: 16)].
	^c!

next
	^ self stream next!

peek
	^ self stream peek!

readAny
	"This is the main entry point for the JSON parser. See also readFrom: on the class side."
	| c |
	self skipWhitespace.
	c := self peek asLowercase.
	c == ${ ifTrue: [self next. ^ self readDictionary].
	c == $[ ifTrue: [self next. ^ self readArray].
	c == $" ifTrue: [self next. ^ self readString].
	c == $t ifTrue: [^ self consume: 'true' returning: true].
	c == $f ifTrue: [^ self consume: 'false' returning: false].
	c == $n ifTrue: [^ self consume: 'null' returning: nil].
	(self class numbersMayContain: c) ifTrue: [^ self readNumber].
	JsonSyntaxError new raiseSignal: 'Unknown Json input'!

readArray
	| a needComma |
	a := OrderedCollection new.
	needComma := false.
	[
		self skipWhitespace.
		self peek == $] ifTrue: [self next. ^ a asArray].
		needComma
			ifTrue: [self peek == $, ifFalse: [JsonSyntaxError new raiseSignal: 'Missing comma'].
					self next.]
			ifFalse: [needComma := true]. 
		a add: self readAny.
	] repeat.!

readDictionary
	| m k v needComma |
	m := Dictionary new.
	needComma := false.
	[
		self skipWhitespace.
		self peek == $} ifTrue: [self next. ^ m].
		needComma
			ifTrue: [self peek == $, ifFalse: [JsonSyntaxError new raiseSignal: 'Missing comma'].
					self next. self skipWhitespace]
			ifFalse: [needComma := true.].
		self next == $" ifFalse: [JsonSyntaxError signal: 'Key in dictionary must be string'].
		k := self readString.
		self skipWhitespace.
		self peek == $: ifFalse: [JsonSyntaxError signal: 'Missing colon'].
		self next.
		v := self readAny.
		m at: k put: v.
	] repeat.!

readNumber
	| acc c |
	acc := WriteStream on: String new.
	[
		c := self peek.
		(c isNil not and: [self class numbersMayContain: c]) ifFalse: [
			[^ acc contents asNumber] on: Error do: [JsonSyntaxError new raiseSignal: 'Invalid number']].
		acc nextPut: c.
		self next.
	] repeat.!

readString
	| s c |
	s := WriteStream on: String new.
	[
		c := self next.
		c == $\
			ifTrue: [s nextPut: self interpretStringEscape.]
			ifFalse: [c == $" ifTrue: [^ s contents.].
					s nextPut: c]
	] repeat.!

skipComment
	
	self peek == $/ ifFalse: [^self].
	self next.
	self peek == $/
		ifTrue: [self skipToEndOfLine]
		ifFalse:
			[self peek == $*
				ifTrue:
					[self next.
					self skipCommentBody]
				ifFalse: [JsonSyntaxError new raiseSignal: 'Invalid comment syntax']]!

skipCommentBody
	[
		[self next == $*] whileFalse.
		self peek == $/
	] whileFalse.
	self next. "skip that last slash"
	self skipWhitespace.!

skipToEndOfLine
	[self peek == Character cr or: [self peek == Character lf]] whileFalse: [self next].
	self skipWhitespace!

skipWhitespace
	[self peek isSeparator] whileTrue: [self next].
	self skipComment.!

stream
	^stream!

stream: anObject
	stream := anObject! !
!JsonReader categoriesFor: #consume:returning:!private! !
!JsonReader categoriesFor: #interpretStringEscape!private! !
!JsonReader categoriesFor: #next!private! !
!JsonReader categoriesFor: #peek!private! !
!JsonReader categoriesFor: #readAny!parsing!public! !
!JsonReader categoriesFor: #readArray!private! !
!JsonReader categoriesFor: #readDictionary!private! !
!JsonReader categoriesFor: #readNumber!private! !
!JsonReader categoriesFor: #readString!private! !
!JsonReader categoriesFor: #skipComment!private! !
!JsonReader categoriesFor: #skipCommentBody!private! !
!JsonReader categoriesFor: #skipToEndOfLine!private! !
!JsonReader categoriesFor: #skipWhitespace!private! !
!JsonReader categoriesFor: #stream!accessing!public! !
!JsonReader categoriesFor: #stream:!accessing!public! !

!JsonReader class methodsFor!

escapeForCharacter: c
	^ CharacterEscapeMap at: c ifAbsent: [nil]!

initialize
	"self initialize"
	CharacterEscapeMap := self initializeCharacterEscapeMap.!

initializeCharacterEscapeMap
	^Dictionary new
		at: $" put: $";
		at: $\ put: $\;
		at: Character backspace put: $b;
		at: Character lf put: $n;
		at: Character newPage put: $f;
		at: Character cr put: $r;
		at: Character tab put: $t;
		yourself!

numbersMayContain: aChar
	
	^aChar isDigit or: ['-+.eE' includes: aChar]!

readFrom: aStream
	^ self new stream: aStream; readAny.!

render: anObject
	| s |
	s := WriteStream on: String new.
	anObject jsonWriteOn: s.
	^ s contents.! !
!JsonReader class categoriesFor: #escapeForCharacter:!constants!public! !
!JsonReader class categoriesFor: #initialize!initializing!public! !
!JsonReader class categoriesFor: #initializeCharacterEscapeMap!initializing!public! !
!JsonReader class categoriesFor: #numbersMayContain:!public!testing! !
!JsonReader class categoriesFor: #readFrom:!instance creation!public! !
!JsonReader class categoriesFor: #render:!instance creation!public! !

JsonSyntaxError guid: (GUID fromString: '{0E9A6415-B919-4AF2-9B64-B0EE6231970F}')!
JsonSyntaxError comment: ''!
!JsonSyntaxError categoriesForClass!Kernel-Objects! !
!JsonSyntaxError methodsFor!

raiseSignal: signalerText
	self signal: signalerText! !
!JsonSyntaxError categoriesFor: #raiseSignal:!private! !

JsonTestCase guid: (GUID fromString: '{9EF05679-537C-4B3B-9F94-56E9CBF09573}')!
JsonTestCase comment: ''!
!JsonTestCase categoriesForClass!Kernel-Objects! !
!JsonTestCase methodsFor!

dictionary: aDictionary contentsEquals: aValue
	"Anwer true if  the receiver and aDictionary contain the same key/values."

	aValue size == aDictionary size
		ifFalse: [ ^false ].

	aValue keysAndValuesDo:
		[ :key :value |
		( aDictionary at: key ifAbsent: [ ^false ] ) = value
			ifFalse: [ ^false ]
		].
	^true!

json: aString contentsEquals: aValue
	self assert: (self dictionary: (JsonReader readFrom: aString readStream) contentsEquals: aValue)!

json: aString equals: aValue
	self assert: (JsonReader readFrom: aString readStream) = aValue.!

render: anObject equals: aString
	self assert: (JsonReader render: anObject) = aString!

testArray
	self json: '[]' equals: #().
	self json: '[[]]' equals: #(#()).
	self json: '[[], []]' equals: #(#() #()).
	self json: '["hi", "there"]' equals: #('hi' 'there').
	self json: '[["a", "b", null]]' equals: #(('a' 'b' nil)).!

testAtomFalse
	self json: 'false' equals: false.
	self json: '  false' equals: false.
	self json: 'false  ' equals: false.
	self json: '  false  ' equals: false.!

testAtomNull
	self json: 'null' equals: nil.
	self json: '  null' equals: nil.
	self json: 'null  ' equals: nil.
	self json: '  null  ' equals: nil.!

testAtomNumber
	self json: '1' equals: 1.
	self json: '0123' equals: 123.
	self json: '1.23e2' equals: 123.
	self json: '-1' equals: -1.
	self json: '-0' equals: 0.
	self json: '[-1.2]' equals: #(-1.2).!

testAtomString
	self json: '"hi"' equals: 'hi'.
	self json: '"\""' equals: '"'.
	self json: '"\\"' equals: '\'.
	self json: '""' equals: ''.!

testAtomTrue
	self json: 'true' equals: true.
	self json: '  true' equals: true.
	self json: 'true  ' equals: true.
	self json: '  true  ' equals: true.!

testDictionary
	self json: '{}' contentsEquals: (Dictionary new).
	self json: '{"a": "a"}' contentsEquals: (Dictionary new at: 'a' put: 'a'; yourself).
	self json: '{"a": [[]]}' contentsEquals: (Dictionary new at: 'a' put: #(#()); yourself).
	self json: '{"a":"b", "b":"a"}' contentsEquals: (Dictionary new at: 'a' put: 'b'; at: 'b' put: 'a'; yourself).!

testWriteString
	self render: '"' equals: '"\""'.
	self render: '\' equals: '"\\"'.
	self render: 'hi' equals: '"hi"'.
	self render: (String withAll: (Array with: $a with: Character lf with: $b)) equals: '"a\nb"'.! !
!JsonTestCase categoriesFor: #dictionary:contentsEquals:!public! !
!JsonTestCase categoriesFor: #json:contentsEquals:!public! !
!JsonTestCase categoriesFor: #json:equals:!public! !
!JsonTestCase categoriesFor: #render:equals:!public! !
!JsonTestCase categoriesFor: #testArray!public! !
!JsonTestCase categoriesFor: #testAtomFalse!public! !
!JsonTestCase categoriesFor: #testAtomNull!public! !
!JsonTestCase categoriesFor: #testAtomNumber!public! !
!JsonTestCase categoriesFor: #testAtomString!public! !
!JsonTestCase categoriesFor: #testAtomTrue!public! !
!JsonTestCase categoriesFor: #testDictionary!public! !
!JsonTestCase categoriesFor: #testWriteString!public! !

"Binary Globals"!

